window.Gleam.initSearch([{"doc":"chip","title":"chip","content":"# chip\n\n[![Package Version](https://img.shields.io/hexpm/v/chip)](https://hex.pm/packages/chip)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/chip/)\n\n\nA local process registry that plays along with Gleam&#39;s [Subject](https://hexdocs.pm/gleam_erlang/gleam/erlang/process.html#Subject) type. \n\nIt can hold a set of subjects to later retrieve individually or dispatch as a group. Will also automatically delist dead processes.\n\n### Example\n\nLets assemble the pieces to build a simple counter actor:\n\n```gleam\nimport gleam/erlang/process\nimport gleam/otp/actor\n\npub opaque type Message {\n  Inc\n  Current(client: process.Subject(Int))\n}\n\npub fn increment(counter) {\n  actor.send(counter, Inc)\n}\n\npub fn current(counter) {\n  actor.call(counter, Current(_), 10)\n}\n\nfn loop(message: Message, count: Int) {\n  case message {\n    Inc -&gt; {\n      actor.Continue(count + 1, option.None)\n    }\n\n    Current(client) -&gt; {\n      process.send(client, count)\n      actor.Continue(count, option.None)\n    }\n  }\n}\n```\n\nWe start our registry and create new instances of a counter:\n\n```gleam\nimport gleam/otp/actor\nimport chip/group\n\npub fn main() {\n  let assert Ok(registry) = chip.start()\n\n  let assert Ok(counter_1) = actor.start(0, loop)\n  let assert Ok(counter_2) = actor.start(0, loop)\n  let assert Ok(counter_3) = actor.start(0, loop)\n\n  chip.register(registry, chip.new(counter_1) |&gt; chip.tag(1))\n  chip.register(registry, chip.new(counter_2) |&gt; chip.tag(2))\n  chip.register(registry, chip.new(counter_3) |&gt; chip.tag(3))\n  \n  process.sleep_forever()\n}\n```\n\nLater, we may retrieve a member:\n \n```gleam\nlet assert Ok(counter) = chip.find(registry, 2)\nlet assert 0 = counter.current(counter)\n```\n\nOr broadcast a message to all members:\n\n```gleam\nchip.dispatch(registry, fn(counter) {\n  actor.increment(counter)\n}) \n\nlet assert Ok(counter) = chip.find(registry, 2)\nlet assert 1 = counter.current(counter)\n```\n\n## The road towards V1\n\nFeature-wise this is near beign complete. Still planning to integrate: \n\n- [x] Adjust the API to be more in-line with other Registry libraries like elixir&#39;s Registry, erlang&#39;s pg or Syn. \n- [x] Document modules.\n- [x] Basic test case scenarios.\n- [X] Should play well with gleam style of supervisors. \n- [ ] Document guides and use-cases. \n- [ ] Build a benchmark that measures performance and memory consuption. \n- [ ] Implement an ETS backend. \n- [ ] Benchmark in-process backend vs ETS backend.\n\n## Previous Art\n\nThis registry takes and combines some ideas from Elixir’s [Registry](https://hexdocs.pm/elixir/Kernel.html), Erlang’s [pg](https://www.erlang.org/doc/apps/kernel/pg.html) and [Syn](https://github.com/ostinelli/syn).\n\n## Alternatives\n\n[Singularity](https://hexdocs.pm/singularity/) is a gleam library that offers registry capabilities but focusing more on singleton actors, therefore it is better suited for keeping track of actors that need to be passed around as configuration through your app. \n\n## Installation\n\n```sh\ngleam add chip\n```\n","url":"index.html"},{"doc":"chip","title":"chip","content":"# General use cases \n\nChip is meant as a general solution for naming and grouping processess, therefore there may be many use cases attached to it. \n\n## As a process Index\n\nProbably one of the most common uses cases for chip will involve looking up for subjects in your system. \n\nFor example, in a web app we may add the registry as part of our app context. \n\n```gleam\npub fn main() {\n  let assert Ok(registry) = chip.start()\n  let self: process.Subject(Nil) = process.new_subject()\n  \n  chip.register(registry, chip.new(self) |&gt; chip.tag(&quot;1st&quot;))\n  chip.register(registry, chip.new(self) |&gt; chip.tag(&quot;2nd&quot;))\n  chip.register(registry, chip.new(self) |&gt; chip.tag(&quot;3rd&quot;))\n  \n  web.server(Context(registry))\n  \n  process.sleep_forever()\n}\n```\n\nThen have access to the subjects at a completely different scope.\n\n```gleam\npub server(request, context) {\n  case request.route {\n    Get, [&quot;/resource/&quot;, id] -&gt; {\n      let result = chip.find(context.registry, id)\n      render(result)\n    }\n  }\n}\n```\n\nOf course, this ability to directly reference subjects is not very useful without a supervision tree, as if the subject dies we can no longer send messages to it. Check the [supervision guideline](chip-as-part-of-a-supervision-tree.html) for more. \n\n## As a dispatcher\n\nOne of the most useful use cases for chip involves dispatching messages to a collection of subjects. \n\nFor example, we may register subjects as part of a group: \n\n```gleam\ntype Group {\n  Even\n  Odd\n}\n\npub fn main() {\n  let assert Ok(registry) = chip.start()\n\n  let assert Ok(store_1) = store.start(1)\n  let assert Ok(store_2) = store.start(2)\n  let assert Ok(store_3) = store.start(3)\n\n  chip.register(registry, chip.new(store_1) |&gt; chip.group(Odd))\n  chip.register(registry, chip.new(store_2) |&gt; chip.group(Even))\n  chip.register(registry, chip.new(store_3) |&gt; chip.group(Odd))\n}\n``` \n\nThen later dispatch messages to all members:\n\n```gleam\nchip.dispatch(registry, fn(store) {\n  store.increment(store)   \n})\n```\n\nOr a sub-group of members:\n\n```gleam\nchip.dispatch_group(registry, Odd, fn(store) {\n  store.increment(store)   \n})\n```\n\nThis is specially useful when wanting to create a PubSub system between subjects. Check the [PubSub Guideline](chip-as-a-local-pubsub.html) for more.  \n\n## As app configuration\n\nOther libraries in the erlang ecosystem like [registry](https://hexdocs.pm/elixir/Kernel.html), [pg](https://www.erlang.org/doc/apps/kernel/pg.html) and [syn](https://github.com/ostinelli/syn) can serve the purpose of storing configuration along processess.\n\nChip may not be well suited for this purpose as it can only store subjects of a single message type. If you need to reference subjects with different message types you may look at the [singularity](https://hexdocs.pm/singularity/) library. Also check the the [wrapping up chip Guideline](wrapping-up-chip.html) for how you may re-purpose a registry for this.\n","url":"guides/general-use-cases.html"},{"doc":"chip","title":"chip","content":"# Chip as part of a Supervision tree\n\nA supervision tree is a strategy used in the wider erlang ecosystem to keep long-running processeess alive. When a process in any part of the tree terminates the supervisor will attempt to restart the process and any subsequent processess after it. Giving our whole system self-healing properties.\n\nLets assume we have the simplest &quot;counter&quot; actor code: \n\n```gleam\npub opaque type Message {\n  Inc\n  Oops\n}\n\nfn loop(message: Message, count: Int) {\n  case message {\n    Inc -&gt; actor.continue(count + 1)\n    Oops -&gt; panic as &quot;unexpected error&quot;\n  }\n}\n```\n\nIntegrating to a supervisor is simple enough if we create a `childspec` for our counter: \n\n```gleam \npub fn main() {\n  let self = process.new_subject()\n\n  let childspec = fn(_param) {\n    // start the counter\n    use counter &lt;- try(actor.start(0, loop))\n    // on success, send counter back to caller.\n    process.send(self, counter)\n    Ok(counter)\n  }\n\n  // start all processess under a supervision tree \n  let assert Ok(_supervisor) =\n    supervisor.start(fn(children) {\n      children\n      |&gt; supervisor.add(supervisor.worker(childspec))\n    })\n\n  do_work(self)\n}\n```\n\nWith the implementation above, we have asured that our supervisor will keep this specific counter spec alive. In the case of unexpected termination the supervisor will re-run the childspec again to restore this and follow-up processess.  \n\nFor example lets assume the counter is terminated at the `do_work` function: \n\n```gleam\nfn do_work(self) {\n  // wait to receive the counter&#39;s subject, and operate on it\n  let assert Ok(counter) = process.receive(self, 50)\n  io.debug(counter)\n\n  // lets attempt to restart it and receive another reference\n  process.send(counter, Oops)\n  let assert Ok(counter) = process.receive(self, 50)\n  io.debug(counter)\n}\n```\n\nThe two subject counter references printed to the terminal are: \n\n```erlang\nSubject(//erl(&lt;0.90.0&gt;), //erl(#Ref&lt;0.3359404744.2770337801.190515&gt;))\nSubject(//erl(&lt;0.91.0&gt;), //erl(#Ref&lt;0.3359404744.2770337801.190559&gt;))\n```\n\nWe can confirm that both process ids `0.90.0` and `0.91.0` are different, therefore when the first counter was terminated the supervisor restarted it and created `0.91.0` in its place.\n\nAll of this requires carrying the `self` reference around the program and knowing when to receive the process in case of failure. We may be completely out of scope. This is where a registry may help. \n\nLets build a `start` specification for our counter actor: \n\n```gleam\npub fn start(registry, tag) {\n  let init = fn() { init(registry, tag) }\n  actor.start_spec(actor.Spec(init: init, init_timeout: 10, loop: loop))\n}\n\nfn init(registry, id) {\n  // Register the counter under an id on initialization\n  chip.register(\n    registry,\n    process.new_subject()\n      |&gt; chip.new()\n      |&gt; chip.tag(id),\n  )\n\n  actor.Ready(0, process.new_selector())\n}\n```\n\nThen integrate chip and our new start function under our supervisor: \n\n```gleam\npub fn main() {\n  let self = process.new_subject()\n\n  let childspec_registry = fn(_param) {\n    use registry &lt;- try(chip.start())\n    // on success, send the registry back to caller.\n    process.send(self, registry)\n    Ok(registry)\n  }\n\n  // Transform initial child parameter to the registry and an id tag\n  let updater_registry = fn(_param, registry) { #(registry, 1) }\n\n  let childspec_counter = fn(param) {\n    // We now receive the registry and initial id\n    let #(registry, id) = param\n    start(registry, id)\n  }\n\n  // Subsequent child counters will increment their id tag\n  let updater_counter = fn(param, _counter) {\n    let #(registry, id) = param\n    #(registry, id + 1)\n  }\n\n  // start all processess under a supervision tree \n  let assert Ok(_supervisor) =\n    supervisor.start(fn(children) {\n      children\n      |&gt; supervisor.add(\n        supervisor.worker(childspec_registry)\n        |&gt; supervisor.returning(updater_registry),\n      )\n      |&gt; supervisor.add(\n        supervisor.worker(childspec_counter)\n        |&gt; supervisor.returning(updater_counter),\n      )\n    })\n\n  // wait to receive the registry&#39;s subject, and operate on it\n  let assert Ok(registry) = process.receive(self, 50)\n  do_work(registry)\n}\n```\n\nIt is quite a bit of extra code and specification, but keep in mind you may move most of this to helpers. Now we can reference the counter from the registry in our `do_work` function.\n\n```gleam\nfn do_work(registry) {\n  // retrieve the counter&#39;s subject, and operate on it\n  let assert Ok(counter) = chip.find(registry, 1)\n  io.debug(counter)\n\n  // lets attempt to restart it and wait for the registry to update\n  process.send(counter, Oops)\n  process.sleep(50)\n  let assert Ok(counter) = chip.find(registry, 1)\n  io.debug(counter)\n}\n```\n\nPrinting us these subjects: \n\n```erlang\nSubject(//erl(&lt;0.91.0&gt;), //erl(#Ref&lt;0.211861637.3582984195.76640&gt;))\nSubject(//erl(&lt;0.92.0&gt;), //erl(#Ref&lt;0.211861637.3582984195.76689&gt;))\n```\n\nGranted the registry didn&#39;t solve all our issues, now we need to pass around a copy of our registry&#39;s subject through our app. And the registry may not yet have registered the new subject (we had to wait a few milliseconds for it to update and restart). \n\nThese issues are out of scope for chip but may be solved through different techniques. For example, top-level processes may use an &quot;app configuration&quot; library that keeps track of singleton processes, if you&#39;d like to re-purpose chip for this please check the [wrapping up chip Guideline](wrapping-up-chip.html) for more.\n","url":"guides/chip-as-part-of-a-supervision-tree.html"},{"doc":"chip","title":"chip","content":"# Chip as a local PubSub system\n\nGleam programs compiling down to erlang are able to take advantage of [erlang&#39;s style of concurrency](https://hexdocs.pm/gleam_erlang/), one of the properties of processess is how they communicate with each other through message passing. \n\nThis property can be used to our advantage to design an even-driven system, where actions are not necesarilly imperative and each component in our system can &quot;react&quot; to external events. One useful tool in event-driven systems is to have a PubSub where we can broadcast messages to whatever processess are subscribed to. \n\nThere are many ways to structure a PubSub system in Gleam so this guide is just a starting point. \n\n## Designing a chat application\n\nLets assume we want to create a chat application. Different clients may subscribe to one of the different chat topics which we will harcode as a type for simplicity: \n\n```gleam\ntype Topic {\n  General\n  Coffee\n  Pets    \n}\n```\n\nOur clients may be any type of subject we would like to register, again for simplicity lets say that our clients are the &quot;main&quot; process itself: \n\n```gleam \npub fn main() {\n  let client_a = process.new_subject()\n  let client_b = process.new_subject()\n  let client_c = process.new_subject()\n}\n```\n\nUpon starting chip we can start subscribing clients to different topics and sending messages: \n\n```gleam\npub fn main() {\n  let client_a = process.new_subject()\n  let client_b = process.new_subject()\n  let client_c = process.new_subject()\n  \n  let assert Ok(pubsub) = chip.start() \n\n  // client A is only interested in general  \n  chip.register(pubsub, chip.new(client_a) |&gt; chip.group(General))\n\n  // client B only cares about coffee\n  chip.register(pubsub, chip.new(client_b) |&gt; chip.group(Coffee))\n\n  // client C wants to be everywhere\n  chip.register(pubsub, chip.new(client_c) |&gt; chip.group(General))\n  chip.register(pubsub, chip.new(client_c) |&gt; chip.group(Coffee))\n  chip.register(pubsub, chip.new(client_c) |&gt; chip.group(Pets))\n  \n  // broadcast a welcome to all subscribed clients\n  task.async(fn() {\n    // lets assume this is the server process broadcasting a welcome message\n    chip.dispatch_group(pubsub, General, fn(client) { process.send(client, &quot;Welcome to General!&quot;) })\n    chip.dispatch_group(pubsub, General, fn(client) { process.send(client, &quot;Please follow the rules&quot;) })\n    chip.dispatch_group(pubsub, General, fn(client) { process.send(client, &quot;and be good with each other :)&quot;) })\n\n    chip.dispatch_group(pubsub, Coffee, fn(client) { process.send(client, &quot;Ice breaker!&quot;)}) \n    chip.dispatch_group(pubsub, Coffee, fn(client) { process.send(client, &quot;Favorite coffee cup?&quot;)}) \n\n    chip.dispatch_group(pubsub, Pets, fn(client) { process.send(client, &quot;Pets!&quot;) })\n  })\n}\n```\n\nIn theory all of our clients should have already received a welcome message in their inbox, but each client is responsible to capture this message so lets build this functionality: \n\n```gleam\nfn listen_for_messages(client, messages) -&gt; List(String) {\n  // this function will listen until messages stop arriving for 100 milliseconds\n  case process.receive(client, 100) {\n    Ok(message) -&gt;\n      // a message was received, capture it and attempt to listen for another message\n      message\n      |&gt; list.prepend(messages, _)\n      |&gt; listen_for_messages(client, _)\n\n    Error(Nil) -&gt;\n      // a message was not received, stop listening and return captured messages in order\n      messages\n      |&gt; list.reverse()\n  }\n}\n```\n\nThen we may use this function to receive messages with each of our clients: \n\n```gleam\n  // client A receives all messages in general\n  let assert [\n    &quot;Welcome to General!&quot;,\n    &quot;Please follow the rules&quot;,\n    &quot;and be good with each other :)&quot;,\n  ] = listen_for_messages(client_a, [])\n  \n  // client B receives all messages in coffee\n  let assert [\n    &quot;Ice breaker!&quot;,\n    &quot;Favorite coffee cup?&quot;,\n  ] = listen_for_messages(client_b, [])\n  \n  // client C receives all messages \n  let assert [\n    &quot;Welcome to General!&quot;,\n    &quot;Please follow the rules&quot;,\n    &quot;and be good with each other :)&quot;,\n    &quot;Ice breaker!&quot;,\n    &quot;Favorite coffee cup?&quot;,\n    &quot;Pets!&quot;,\n  ] = listen_for_messages(client_c, [])\n```\n\nAnd with this we have all the components required for a very basic PubSub system that does subscription and topics. \n\n## A note on data modelling\n\nWe may be tempted to look at the example above and try to modularize it in more discreete, single responsability modules. Keep in mind that PubSub systems may spiral out and grow through the entire system, this will likely not be a problem as Gleam (the language) really doesn&#39;t like circular dependencies.\n\nFor example. If you have a `Server` module and a `PubSub` module the dependency (likely) will go this direction: \n\n```\nServer  -- uses --&gt; PubSub\n```  \n\nBut lets say that `Server` does define an `Event` type, which is used by the `PubSub` module, now the dependency goes both ways: \n\n```\nServer &lt;-- uses --&gt; PubSub\n```  \n\nThere are a couple of ways to avoid this: \n\n* Divide shared types, in this case `Event` in their own module.  \n* Take advantage of generics so you don&#39;t have to be bound to specific types.\n* Take advantage of callback so you don&#39;t have to be bound to specific behaviour.\n\nSometimes managing the above is quite a headache and not worth it, specially when your domain is not so well defined. \n\n## Not all servers speak in Strings\n\nSo far we have assumed that our server, client and PubSub all speak in `String` messages:  \n\n```gleam\n// Server speaks in String\nprocess.send(client, &quot;Welcome to General!&quot;)\n\n// Client speaks in String\nlet client_a: process.Subject(String) = process.new_subject()\n\n// PubSub speaks in String\nlet assert Ok(pubsub) = chip.start()\nlet pubsub: process.Subject(chip.Message(String, Nil, Topic))\n```\n\nWhat if a single client was listening to multiple servers? Lets say that some servers communicate in `Java` language for sharing coffee jargon: \n\n```gleam\ntype Java {\n  Brew\n  Drip    \n  Temp\n}\n```\n\nWhile others used `Pet` for added goofiness: \n\n```gleam\ntype Pet {\n  Woof\n  Meow\n  Splash    \n}\n```\n\nThese new event types are incompatible to our PubSub but this doesn&#39;t mean we can&#39;t design a client that listents to theset types of messages; we just need to modify the approach. \n\nWe can create different subjects with their own types: \n\n```gleam \nlet client_a: process.Subject(String) = process.new_subject()\nlet client_b: process.Subject(Java) = process.new_subject()\nlet client_b: process.Subject(Pet) = process.new_subject()\n```\n\nAnd (because of Chip&#39;s limitations) different pubsubs with their own types also: \n\n```gleam\ntype PubSub(message, tag) =\n  process.Subject(chip.Message(message, tag, Topic))\n\nlet assert Ok(general) = chip.start()\nlet general: PubSub(String, tag) = general\n\nlet assert Ok(java) = chip.start()\nlet java: PubSub(Java, tag) = java\n\nlet assert Ok(pet) = chip.start()\nlet java: PubSub(Pet, tag) = pet\n```\n\nEach pubsub above is designed for a specific server that sends its own type of events: \n\n```gleam\n// the server broadcasts coded messages \ntask.async(fn() {\n  chip.dispatch(general, fn(client) {\n    process.send(client, &quot;How&#39;s everyone&quot;)\n  })\n  \n  chip.dispatch(java, fn(client) {\n    process.send(client, Brew)\n  })\n  \n  chip.dispatch(pet, fn(client) {\n    process.send(client, Meow)\n  })\n})\n```\n\nAnd we can treat our 3 subjects above as a single client by taking advantage of process selectors, lets modify a bit our `listen_for_messages` function: \n\n```gleam\nfn listen_for_protocol(selector, messages) -&gt; List(String) {\n  case process.select(selector, 100) {\n    Ok(message) -&gt;\n      message\n      |&gt; list.prepend(messages, _)\n      |&gt; listen_for_protocol(selector, _)\n\n    Error(Nil) -&gt;\n      messages\n      |&gt; list.reverse()\n  }\n}\n```\n\nFinally using a selector, receive all messages as a single client: \n\n```gleam\n// then receive as a single client\nlet assert [&quot;How&#39;s everyone&quot;, &quot;brewing&quot;, &quot;meoww&quot;] =\n  process.new_selector()\n  |&gt; process.selecting(client_a, identity)\n  |&gt; process.selecting(client_b, protocol_java)\n  |&gt; process.selecting(client_c, protocol_pet)\n  |&gt; listen_for_protocol([])\n\nfn identity(x: x) -&gt; x {\n  x    \n}\n\nfn protocol_java(java: Java) -&gt; String {\n  case java {\n    Drip -&gt; &quot;to drip&quot;\n    Brew -&gt; &quot;brewing&quot;\n    Temp -&gt; &quot;at temp&quot;\n  }\n}\n\nfn protocol_pet(noise: Pet) -&gt; String {\n  case noise {\n    Woof -&gt; &quot;woof!&quot;\n    Meow -&gt; &quot;meoww&quot;\n    Splash -&gt; &quot;&quot;\n  }\n}\n```\n\nAnd this would be a way of managing multiple message types from different sources (servers, pubsubs) with a single client. \n","url":"guides/chip-as-a-local-pubsub.html"},{"doc":"chip","title":"chip","content":"# Wrapping up chip\n\n","url":"guides/wrapping-up-chip.html"},{"doc":"chip","title":"Chip","content":"pub opaque type Chip(msg, tag, group)\n A &quot;chip&quot; used for registration. Check the [new](#new) function.\n\nSynonyms:\nchip.Chip\nchip Chip","url":"chip.html#Chip"},{"doc":"chip","title":"Message","content":"pub opaque type Message(msg, tag, group)\n Chip&#39;s internal message type.\n\nSynonyms:\nchip.Message\nchip Message","url":"chip.html#Message"},{"doc":"chip","title":"Registry","content":"pub type Registry(msg, tag, group) =\n  process.Subject(Message(msg, tag, group))\n An shorter alias for the registry&#39;s Subject. \n \n Sometimes, when building out your system it may be useful to state the Registry&#39;s types.\n \n ## Example\n \n \n let assert Ok(registry) = chip.start()\n let registry: chip.Registry(Event, Id, Topic)\n \n \n Which is equivalent to: \n \n \n let assert Ok(registry) = chip.start()\n let registry: process.Subject(chip.Message(Event, Id, Topic))\n \n \n By specifying the types we can document the kind of registry we are working with.\n\nSynonyms:\nchip.Registry\nchip Registry","url":"chip.html#Registry"},{"doc":"chip","title":"dispatch","content":"pub fn dispatch(\n  registry: Subject(Message(a, b, c)),\n  callback: fn(Subject(a)) -&gt; d,\n) -&gt; Nil\n Applies a callback over all registered Subjects.\n \n ## Example\n \n \n chip.dispatch(registry, fn(subject) { \n   process.send(subject, message)\n })\n \nSynonyms:\nchip.dispatch\nchip dispatch","url":"chip.html#dispatch"},{"doc":"chip","title":"dispatch_group","content":"pub fn dispatch_group(\n  registry: Subject(Message(a, b, c)),\n  group: c,\n  callback: fn(Subject(a)) -&gt; d,\n) -&gt; Nil\n Applies a callback over a group.\n \n ## Example\n \n \n chip.dispatch_group(registry, Pets, fn(subject) { \n   process.send(subject, message)\n })\n \nSynonyms:\nchip.dispatch_group\nchip dispatch_group","url":"chip.html#dispatch_group"},{"doc":"chip","title":"find","content":"pub fn find(\n  registry: Subject(Message(a, b, c)),\n  tag: b,\n) -&gt; Result(Subject(a), Nil)\n Retrieves a tagged subject.\n \n ## Example\n \n \n let assert Ok(subject) = chip.find(registry, &quot;Luis&quot;)\n \nSynonyms:\nchip.find\nchip find","url":"chip.html#find"},{"doc":"chip","title":"group","content":"pub fn group(\n  registrant: Chip(a, b, c),\n  group: c,\n) -&gt; Chip(a, b, c)\n Adds the &quot;chip&quot; under a group. \n \n ## Example\n \n \n chip.new(subject)\n |&gt; chip.group(General) \n \nSynonyms:\nchip.group\nchip group","url":"chip.html#group"},{"doc":"chip","title":"new","content":"pub fn new(subject: Subject(a)) -&gt; Chip(a, b, c)\n Creates a new &quot;chip&quot; that can be tagged, grouped and registered. \n \n ## Example\n \n \n chip.new(subject)\n \nSynonyms:\nchip.new\nchip new","url":"chip.html#new"},{"doc":"chip","title":"register","content":"pub fn register(\n  registry: Subject(Message(a, b, c)),\n  registrant: Chip(a, b, c),\n) -&gt; Nil\n Registers a `Registrant`. \n \n ## Example\n \n \n let assert Ok(registry) = chip.start()\n \n chip.new(subject)\n |&gt; chip.register(registry, _)\n \n \n `Registrant` may be registered under a tag or group.\n \n \n let assert Ok(registry) = chip.start()\n \n chip.new(subject)\n |&gt; chip.tag(&quot;Francisco&quot;) \n |&gt; chip.group(Coffee) \n |&gt; chip.register(registry, _)\n \n \n You may register any subject at any point in time but usually keeping it under the initialization\n step of your process (like an Actor&#39;s `init` callback) will keep things organized and tidy. \nSynonyms:\nchip.register\nchip register","url":"chip.html#register"},{"doc":"chip","title":"start","content":"pub fn start() -&gt; Result(Subject(Message(a, b, c)), StartError)\n Starts the registry.\n \n ## Example\n \n \n &gt; chip.start()\n \nSynonyms:\nchip.start\nchip start","url":"chip.html#start"},{"doc":"chip","title":"tag","content":"pub fn tag(registrant: Chip(a, b, c), tag: b) -&gt; Chip(a, b, c)\n Adds a unique tag to a &quot;chip&quot;, it will overwrite any previous subject under the same tag.\n \n ## Example\n \n \n chip.new(subject)\n |&gt; chip.tag(&quot;Luis&quot;) \n \nSynonyms:\nchip.tag\nchip tag","url":"chip.html#tag"},{"doc":"chip","title":"chip","content":" Chip is a local process registry that plays along with Gleam&#39;s `Subject` type for referencing\n erlang processes. It can hold to a set of subjects to later reference individually or dispatch \n a callback as a group. Will also automatically delist dead processes.","url":"chip.html"}]);